@@ -19,24 +19,29 @@
 
 import logging
 import os
+from unittest import mock
 
 import fixtures
-import mock
 from oslo_config import cfg
-from oslotest import base as oslo_test
-import six
 import sqlalchemy
+from sqlalchemy.engine import base as base_engine
+from sqlalchemy import exc
+from sqlalchemy.pool import NullPool
+from sqlalchemy import sql
 from sqlalchemy import Column, MetaData, Table
-from sqlalchemy.engine import url
 from sqlalchemy import Integer, String
-from sqlalchemy.ext.declarative import declarative_base
+from sqlalchemy.orm import declarative_base
 
 from oslo_db import exception
 from oslo_db import options as db_options
+from oslo_db.sqlalchemy import compat
+from oslo_db.sqlalchemy import enginefacade
 from oslo_db.sqlalchemy import engines
 from oslo_db.sqlalchemy import models
 from oslo_db.sqlalchemy import session
-from oslo_db.sqlalchemy import test_base
+from oslo_db.sqlalchemy import utils
+from oslo_db.tests import base as test_base
+from oslo_db.tests.sqlalchemy import base as db_test_base
 
 
 BASE = declarative_base()
@@ -51,23 +56,22 @@ class RegexpTable(BASE, models.ModelBase):
     bar = Column(String(255))
 
 
-class RegexpFilterTestCase(test_base.DbTestCase):
+class RegexpFilterTestCase(db_test_base._DbTestCase):
 
     def setUp(self):
         super(RegexpFilterTestCase, self).setUp()
         meta = MetaData()
-        meta.bind = self.engine
         test_table = Table(_REGEXP_TABLE_NAME, meta,
                            Column('id', Integer, primary_key=True,
                                   nullable=False),
                            Column('bar', String(255)))
-        test_table.create()
-        self.addCleanup(test_table.drop)
+        test_table.create(self.engine)
+        self.addCleanup(test_table.drop, self.engine)
 
     def _test_regexp_filter(self, regexp, expected):
-        _session = self.sessionmaker()
-        with _session.begin():
-            for i in ['10', '20', u'♥']:
+        with enginefacade.writer.using(db_test_base.context):
+            _session = db_test_base.context.session
+            for i in ['10', '20', '♥']:
                 tbl = RegexpTable()
                 tbl.update({'bar': i})
                 tbl.save(session=_session)
@@ -83,13 +87,13 @@ class RegexpFilterTestCase(test_base.DbTestCase):
         self._test_regexp_filter('11', [])
 
     def test_regexp_filter_unicode(self):
-        self._test_regexp_filter(u'♥', [u'♥'])
+        self._test_regexp_filter('♥', ['♥'])
 
     def test_regexp_filter_unicode_nomatch(self):
-        self._test_regexp_filter(u'♦', [])
+        self._test_regexp_filter('♦', [])
 
 
-class SQLiteSavepointTest(test_base.DbTestCase):
+class SQLiteSavepointTest(db_test_base._DbTestCase):
     def setUp(self):
         super(SQLiteSavepointTest, self).setUp()
         meta = MetaData()
@@ -109,16 +113,22 @@ class SQLiteSavepointTest(test_base.DbTestCase):
         )
         self.assertEqual(
             [(1, 'data 1')],
-            self.engine.execute(
+            conn.execute(
                 self.test_table.select().
                 order_by(self.test_table.c.id)
             ).fetchall()
         )
         trans.rollback()
-        self.assertEqual(
-            0,
-            self.engine.scalar(self.test_table.count())
-        )
+
+        with self.engine.connect() as conn:
+            self.assertEqual(
+                0,
+                conn.scalar(
+                    sqlalchemy.select(
+                        sqlalchemy.func.count(self.test_table.c.id),
+                    ).select_from(self.test_table)
+                )
+            )
 
     def test_savepoint_middle(self):
         with self.engine.begin() as conn:
@@ -139,13 +149,13 @@ class SQLiteSavepointTest(test_base.DbTestCase):
                 {'data': 'data 3'}
             )
 
-        self.assertEqual(
-            [(1, 'data 1'), (2, 'data 3')],
-            self.engine.execute(
-                self.test_table.select().
-                order_by(self.test_table.c.id)
-            ).fetchall()
-        )
+            self.assertEqual(
+                [(1, 'data 1'), (2, 'data 3')],
+                conn.execute(
+                    self.test_table.select().
+                    order_by(self.test_table.c.id)
+                ).fetchall()
+            )
 
     def test_savepoint_beginning(self):
         with self.engine.begin() as conn:
@@ -161,13 +171,13 @@ class SQLiteSavepointTest(test_base.DbTestCase):
                 {'data': 'data 2'}
             )
 
-        self.assertEqual(
-            [(1, 'data 2')],
-            self.engine.execute(
-                self.test_table.select().
-                order_by(self.test_table.c.id)
-            ).fetchall()
-        )
+            self.assertEqual(
+                [(1, 'data 2')],
+                conn.execute(
+                    self.test_table.select().
+                    order_by(self.test_table.c.id)
+                ).fetchall()
+            )
 
 
 class FakeDBAPIConnection(object):
@@ -196,33 +206,100 @@ class ProgrammingError(Exception):
     pass
 
 
-class FakeDB2Engine(object):
+class QueryParamTest(db_test_base._DbTestCase):
+    def _fixture(self):
+        from sqlalchemy import create_engine
 
-    class Dialect(object):
+        def _mock_create_engine(*arg, **kw):
+            return create_engine("sqlite://")
 
-        def is_disconnect(self, e, *args):
-            expected_error = ('SQL30081N: DB2 Server connection is no longer '
-                              'active')
-            return (str(e) == expected_error)
+        return mock.patch(
+            "oslo_db.sqlalchemy.engines.sqlalchemy.create_engine",
+            side_effect=_mock_create_engine)
 
-    dialect = Dialect()
-    name = 'ibm_db_sa'
+    def _normalize_query_dict(self, qdict):
+        # SQLAlchemy 1.4 returns url.query as:
+        # immutabledict({k1: v1, k2: (v2a, v2b, ...), ...})
+        # that is with tuples not lists for multiparams
 
-    def dispose(self):
-        pass
+        return {
+            k: list(v) if isinstance(v, tuple) else v
+            for k, v in qdict.items()
+        }
+
+    def test_add_assorted_params(self):
+        with self._fixture() as ce:
+            engines.create_engine(
+                "mysql+pymysql://foo:bar@bat",
+                connection_parameters="foo=bar&bat=hoho&bat=param2")
+
+        self.assertEqual(
+            self._normalize_query_dict(ce.mock_calls[0][1][0].query),
+            {'bat': ['hoho', 'param2'], 'foo': 'bar'}
+        )
+
+    def test_add_no_params(self):
+        with self._fixture() as ce:
+            engines.create_engine(
+                "mysql+pymysql://foo:bar@bat")
+
+        self.assertEqual(
+            ce.mock_calls[0][1][0].query,
+            self._normalize_query_dict({})
+        )
+
+    def test_combine_params(self):
+        with self._fixture() as ce:
+            engines.create_engine(
+                "mysql+pymysql://foo:bar@bat/"
+                "?charset=utf8&param_file=tripleo.cnf",
+                connection_parameters="plugin=sqlalchemy_collectd&"
+                                      "collectd_host=127.0.0.1&"
+                                      "bind_host=192.168.1.5")
+
+        self.assertEqual(
+            self._normalize_query_dict(ce.mock_calls[0][1][0].query),
+            {
+                'bind_host': '192.168.1.5',
+                'charset': 'utf8',
+                'collectd_host': '127.0.0.1',
+                'param_file': 'tripleo.cnf',
+                'plugin': 'sqlalchemy_collectd'
+            }
+        )
+
+    def test_combine_multi_params(self):
+        with self._fixture() as ce:
+            engines.create_engine(
+                "mysql+pymysql://foo:bar@bat/"
+                "?charset=utf8&param_file=tripleo.cnf&plugin=connmon",
+                connection_parameters="plugin=sqlalchemy_collectd&"
+                                      "collectd_host=127.0.0.1&"
+                                      "bind_host=192.168.1.5")
+
+        self.assertEqual(
+            self._normalize_query_dict(ce.mock_calls[0][1][0].query),
+            {
+                'bind_host': '192.168.1.5',
+                'charset': 'utf8',
+                'collectd_host': '127.0.0.1',
+                'param_file': 'tripleo.cnf',
+                'plugin': ['connmon', 'sqlalchemy_collectd']
+            }
+        )
 
 
-class MySQLDefaultModeTestCase(test_base.MySQLOpportunisticTestCase):
+class MySQLDefaultModeTestCase(db_test_base._MySQLOpportunisticTestCase):
     def test_default_is_traditional(self):
         with self.engine.connect() as conn:
             sql_mode = conn.execute(
-                "SHOW VARIABLES LIKE 'sql_mode'"
+                sql.text("SHOW VARIABLES LIKE 'sql_mode'")
             ).first()[1]
 
-        self.assertTrue("TRADITIONAL" in sql_mode)
+        self.assertIn("TRADITIONAL", sql_mode)
 
 
-class MySQLModeTestCase(test_base.MySQLOpportunisticTestCase):
+class MySQLModeTestCase(db_test_base._MySQLOpportunisticTestCase):
 
     def __init__(self, *args, **kwargs):
         super(MySQLModeTestCase, self).__init__(*args, **kwargs)
@@ -241,20 +318,22 @@ class MySQLModeTestCase(test_base.MySQLOpportunisticTestCase):
         self.test_table = Table(_TABLE_NAME + "mode", meta,
                                 Column('id', Integer, primary_key=True),
                                 Column('bar', String(255)))
-        self.test_table.create(self.connection)
+        with self.connection.begin():
+            self.test_table.create(self.connection)
 
         def cleanup():
-            self.test_table.drop(self.connection)
+            with self.connection.begin():
+                self.test_table.drop(self.connection)
             self.connection.close()
             mode_engine.dispose()
+
         self.addCleanup(cleanup)
 
     def _test_string_too_long(self, value):
         with self.connection.begin():
-            self.connection.execute(self.test_table.insert(),
-                                    bar=value)
+            self.connection.execute(self.test_table.insert(), {'bar': value})
             result = self.connection.execute(self.test_table.select())
-            return result.fetchone()['bar']
+            return result.fetchone().bar
 
     def test_string_too_long(self):
         value = 'a' * 512
@@ -291,7 +370,7 @@ class MySQLTraditionalModeTestCase(MySQLStrictAllTablesModeTestCase):
         self.mysql_mode = 'TRADITIONAL'
 
 
-class EngineFacadeTestCase(oslo_test.BaseTestCase):
+class EngineFacadeTestCase(test_base.BaseTestCase):
     def setUp(self):
         super(EngineFacadeTestCase, self).setUp()
 
@@ -310,9 +389,8 @@ class EngineFacadeTestCase(oslo_test.BaseTestCase):
         self.assertIsNot(ses1, ses2)
 
     def test_get_session_arguments_override_default_settings(self):
-        ses = self.facade.get_session(autocommit=False, expire_on_commit=True)
+        ses = self.facade.get_session(expire_on_commit=True)
 
-        self.assertFalse(ses.autocommit)
         self.assertTrue(ses.expire_on_commit)
 
     @mock.patch('oslo_db.sqlalchemy.orm.get_maker')
@@ -331,17 +409,16 @@ class EngineFacadeTestCase(oslo_test.BaseTestCase):
         for optname, optvalue in overrides.items():
             conf.set_override(optname, optvalue, group='database')
 
-        session.EngineFacade.from_config(conf,
-                                         autocommit=False,
-                                         expire_on_commit=True)
+        session.EngineFacade.from_config(conf, expire_on_commit=True)
 
         create_engine.assert_called_once_with(
             sql_connection='sqlite:///:memory:',
             connection_debug=100,
             max_pool_size=10,
             mysql_sql_mode='TRADITIONAL',
+            mysql_wsrep_sync_wait=None,
             sqlite_fk=False,
-            idle_timeout=mock.ANY,
+            connection_recycle_time=mock.ANY,
             retry_interval=mock.ANY,
             max_retries=mock.ANY,
             max_overflow=mock.ANY,
@@ -351,11 +428,12 @@ class EngineFacadeTestCase(oslo_test.BaseTestCase):
             thread_checkin=mock.ANY,
             json_serializer=None,
             json_deserializer=None,
+            connection_parameters='',
             logging_name=mock.ANY,
         )
-        get_maker.assert_called_once_with(engine=create_engine(),
-                                          autocommit=False,
-                                          expire_on_commit=True)
+        get_maker.assert_called_once_with(
+            engine=create_engine(), expire_on_commit=True,
+        )
 
     def test_slave_connection(self):
         paths = self.create_tempfiles([('db.master', ''), ('db.slave', '')],
@@ -395,58 +473,106 @@ class EngineFacadeTestCase(oslo_test.BaseTestCase):
         self.assertEqual(master_path, str(slave_session.bind.url))
 
 
-class SQLiteConnectTest(oslo_test.BaseTestCase):
+class SQLiteConnectTest(test_base.BaseTestCase):
 
     def _fixture(self, **kw):
         return session.create_engine("sqlite://", **kw)
 
     def test_sqlite_fk_listener(self):
         engine = self._fixture(sqlite_fk=True)
-        self.assertEqual(
-            1,
-            engine.scalar("pragma foreign_keys")
-        )
+        with engine.connect() as conn:
+            self.assertEqual(
+                1,
+                conn.execute(
+                    sql.text('pragma foreign_keys')
+                ).scalars().first(),
+            )
 
         engine = self._fixture(sqlite_fk=False)
 
-        self.assertEqual(
-            0,
-            engine.scalar("pragma foreign_keys")
-        )
+        with engine.connect() as conn:
+            self.assertEqual(
+                0,
+                conn.execute(
+                    sql.text('pragma foreign_keys')
+                ).scalars().first(),
+            )
 
     def test_sqlite_synchronous_listener(self):
         engine = self._fixture()
 
         # "The default setting is synchronous=FULL." (e.g. 2)
         # http://www.sqlite.org/pragma.html#pragma_synchronous
-        self.assertEqual(
-            2,
-            engine.scalar("pragma synchronous")
-        )
+        with engine.connect() as conn:
+            self.assertEqual(
+                2,
+                conn.execute(sql.text('pragma synchronous')).scalars().first(),
+            )
 
         engine = self._fixture(sqlite_synchronous=False)
 
-        self.assertEqual(
-            0,
-            engine.scalar("pragma synchronous")
-        )
+        with engine.connect() as conn:
+            self.assertEqual(
+                0,
+                conn.execute(sql.text('pragma synchronous')).scalars().first(),
+            )
 
 
-class MysqlConnectTest(test_base.MySQLOpportunisticTestCase):
+class MysqlConnectTest(db_test_base._MySQLOpportunisticTestCase):
 
-    def _fixture(self, sql_mode):
-        return session.create_engine(self.engine.url, mysql_sql_mode=sql_mode)
+    def _fixture(self, sql_mode=None, mysql_wsrep_sync_wait=None):
+
+        kw = {}
+        if sql_mode is not None:
+            kw["mysql_sql_mode"] = sql_mode
+        if mysql_wsrep_sync_wait is not None:
+            kw["mysql_wsrep_sync_wait"] = mysql_wsrep_sync_wait
+
+        return session.create_engine(self.engine.url, **kw)
 
     def _assert_sql_mode(self, engine, sql_mode_present, sql_mode_non_present):
-        mode = engine.execute("SHOW VARIABLES LIKE 'sql_mode'").fetchone()[1]
-        self.assertTrue(
-            sql_mode_present in mode
+        with engine.connect() as conn:
+            mode = conn.execute(
+                sql.text("SHOW VARIABLES LIKE 'sql_mode'")
+            ).fetchone()[1]
+        self.assertIn(
+            sql_mode_present, mode
         )
         if sql_mode_non_present:
-            self.assertTrue(
-                sql_mode_non_present not in mode
+            self.assertNotIn(
+                sql_mode_non_present, mode
             )
 
+    def test_mysql_wsrep_sync_wait_listener(self):
+        with self.engine.connect() as conn:
+            try:
+                conn.execute(
+                    sql.text("show variables like '%wsrep_sync_wait%'")
+                ).scalars(1).one()
+            except exc.NoResultFound:
+                self.skipTest("wsrep_sync_wait option is not available")
+
+        engine = self._fixture()
+
+        with engine.connect() as conn:
+            self.assertEqual(
+                "0",
+                conn.execute(
+                    sql.text("show variables like '%wsrep_sync_wait%'")
+                ).scalars(1).one(),
+            )
+
+        for wsrep_val in (2, 1, 5):
+            engine = self._fixture(mysql_wsrep_sync_wait=wsrep_val)
+
+            with engine.connect() as conn:
+                self.assertEqual(
+                    str(wsrep_val),
+                    conn.execute(
+                        sql.text("show variables like '%wsrep_sync_wait%'")
+                    ).scalars(1).one(),
+                )
+
     def test_set_mode_traditional(self):
         engine = self._fixture(sql_mode='TRADITIONAL')
         self._assert_sql_mode(engine, "TRADITIONAL", "ANSI")
@@ -462,8 +588,10 @@ class MysqlConnectTest(test_base.MySQLOpportunisticTestCase):
         # get the GLOBAL sql_mode, not the @@SESSION, so that
         # we get what is configured for the MySQL database, as opposed
         # to what our own session.create_engine() has set it to.
-        expected = self.engine.execute(
-            "SELECT @@GLOBAL.sql_mode").scalar()
+        with self.engine.connect() as conn:
+            expected = conn.execute(
+                sql.text("SELECT @@GLOBAL.sql_mode")
+            ).scalar()
 
         engine = self._fixture(sql_mode=None)
         self._assert_sql_mode(engine, expected, None)
@@ -515,8 +643,10 @@ class MysqlConnectTest(test_base.MySQLOpportunisticTestCase):
 
         engine = self._fixture(sql_mode='TRADITIONAL')
 
-        actual_mode = engine.execute(
-            "SHOW VARIABLES LIKE 'sql_mode'").fetchone()[1]
+        with engine.connect() as conn:
+            actual_mode = conn.execute(
+                sql.text("SHOW VARIABLES LIKE 'sql_mode'")
+            ).fetchone()[1]
 
         self.assertIn('MySQL server mode set to %s' % actual_mode,
                       log.output)
@@ -552,7 +682,7 @@ class MysqlConnectTest(test_base.MySQLOpportunisticTestCase):
                          log.output)
 
 
-class CreateEngineTest(oslo_test.BaseTestCase):
+class CreateEngineTest(test_base.BaseTestCase):
     """Test that dialect-specific arguments/ listeners are set up correctly.
 
     """
@@ -563,94 +693,92 @@ class CreateEngineTest(oslo_test.BaseTestCase):
 
     def test_queuepool_args(self):
         engines._init_connection_args(
-            url.make_url("mysql+pymysql://u:p@host/test"), self.args,
-            max_pool_size=10, max_overflow=10)
+            utils.make_url("mysql+pymysql://u:p@host/test"),
+            self.args,
+            {'max_pool_size': 10, 'max_overflow': 10},
+        )
         self.assertEqual(10, self.args['pool_size'])
         self.assertEqual(10, self.args['max_overflow'])
 
     def test_sqlite_memory_pool_args(self):
         for _url in ("sqlite://", "sqlite:///:memory:"):
             engines._init_connection_args(
-                url.make_url(_url), self.args,
-                max_pool_size=10, max_overflow=10)
+                utils.make_url(_url),
+                self.args,
+                {'max_pool_size': 10, 'max_overflow': 10},
+            )
 
-            # queuepool arguments are not peresnet
-            self.assertTrue(
-                'pool_size' not in self.args)
-            self.assertTrue(
-                'max_overflow' not in self.args)
+            # queuepool arguments are not present
+            self.assertNotIn('pool_size', self.args)
+            self.assertNotIn('max_overflow', self.args)
 
             self.assertEqual(False,
                              self.args['connect_args']['check_same_thread'])
 
             # due to memory connection
-            self.assertTrue('poolclass' in self.args)
+            self.assertIn('poolclass', self.args)
 
     def test_sqlite_file_pool_args(self):
         engines._init_connection_args(
-            url.make_url("sqlite:///somefile.db"), self.args,
-            max_pool_size=10, max_overflow=10)
+            utils.make_url("sqlite:///somefile.db"),
+            self.args,
+            {'max_pool_size': 10, 'max_overflow': 10},
+        )
 
         # queuepool arguments are not peresnet
-        self.assertTrue('pool_size' not in self.args)
-        self.assertTrue(
-            'max_overflow' not in self.args)
+        self.assertNotIn('pool_size', self.args)
+        self.assertNotIn(
+            'max_overflow', self.args)
 
         self.assertFalse(self.args['connect_args'])
 
-        # NullPool is the default for file based connections,
-        # no need to specify this
-        self.assertTrue('poolclass' not in self.args)
+        if not compat.sqla_2:
+            # NullPool is the default for file based connections,
+            # no need to specify this
+            self.assertNotIn('poolclass', self.args)
+        else:
+            self.assertIs(self.args["poolclass"], NullPool)
 
     def _test_mysql_connect_args_default(self, connect_args):
-        if six.PY3:
-            self.assertEqual({'charset': 'utf8', 'use_unicode': 1},
-                             connect_args)
-        else:
-            self.assertEqual({'charset': 'utf8', 'use_unicode': 0},
-                             connect_args)
+        self.assertEqual({'charset': 'utf8', 'use_unicode': 1},
+                         connect_args)
 
     def test_mysql_connect_args_default(self):
         engines._init_connection_args(
-            url.make_url("mysql://u:p@host/test"), self.args)
-        self._test_mysql_connect_args_default(self.args['connect_args'])
-
-    def test_mysql_oursql_connect_args_default(self):
-        engines._init_connection_args(
-            url.make_url("mysql+oursql://u:p@host/test"), self.args)
+            utils.make_url("mysql://u:p@host/test"), self.args, {})
         self._test_mysql_connect_args_default(self.args['connect_args'])
 
     def test_mysql_pymysql_connect_args_default(self):
         engines._init_connection_args(
-            url.make_url("mysql+pymysql://u:p@host/test"), self.args)
+            utils.make_url("mysql+pymysql://u:p@host/test"), self.args, {})
         self.assertEqual({'charset': 'utf8'}, self.args['connect_args'])
 
     def test_mysql_mysqldb_connect_args_default(self):
         engines._init_connection_args(
-            url.make_url("mysql+mysqldb://u:p@host/test"), self.args)
+            utils.make_url("mysql+mysqldb://u:p@host/test"), self.args, {})
         self._test_mysql_connect_args_default(self.args['connect_args'])
 
     def test_postgresql_connect_args_default(self):
         engines._init_connection_args(
-            url.make_url("postgresql://u:p@host/test"), self.args)
+            utils.make_url("postgresql://u:p@host/test"), self.args, {})
         self.assertEqual('utf8', self.args['client_encoding'])
         self.assertFalse(self.args['connect_args'])
 
     def test_mysqlconnector_raise_on_warnings_default(self):
         engines._init_connection_args(
-            url.make_url("mysql+mysqlconnector://u:p@host/test"),
-            self.args)
+            utils.make_url("mysql+mysqlconnector://u:p@host/test"),
+            self.args, {})
         self.assertEqual(False, self.args['connect_args']['raise_on_warnings'])
 
     def test_mysqlconnector_raise_on_warnings_override(self):
         engines._init_connection_args(
-            url.make_url(
+            utils.make_url(
                 "mysql+mysqlconnector://u:p@host/test"
                 "?raise_on_warnings=true"),
-            self.args
+            self.args, {}
         )
 
-        self.assertFalse('raise_on_warnings' in self.args['connect_args'])
+        self.assertNotIn('raise_on_warnings', self.args['connect_args'])
 
     def test_thread_checkin(self):
         with mock.patch("sqlalchemy.event.listens_for"):
@@ -663,8 +791,54 @@ class CreateEngineTest(oslo_test.BaseTestCase):
             engines._thread_yield
         )
 
+    def test_warn_on_missing_driver(self):
 
-class ProcessGuardTest(test_base.DbTestCase):
+        warnings = mock.Mock()
+
+        def warn_interpolate(msg, args):
+            # test the interpolation itself to ensure the password
+            # is concealed
+            warnings.warning(msg % (args, ))
+
+        with mock.patch(
+                "oslo_db.sqlalchemy.engines.LOG.warning",
+                warn_interpolate):
+
+            engines._vet_url(
+                utils.make_url("mysql://scott:tiger@some_host/some_db"))
+            engines._vet_url(utils.make_url(
+                "mysql+mysqldb://scott:tiger@some_host/some_db"))
+            engines._vet_url(utils.make_url(
+                "mysql+pymysql://scott:tiger@some_host/some_db"))
+            engines._vet_url(utils.make_url(
+                "postgresql+psycopg2://scott:tiger@some_host/some_db"))
+            engines._vet_url(utils.make_url(
+                "postgresql://scott:tiger@some_host/some_db"))
+
+        self.assertEqual(
+            [
+                mock.call.warning(
+                    "URL mysql://scott:***@some_host/some_db does not contain "
+                    "a '+drivername' portion, "
+                    "and will make use of a default driver.  "
+                    "A full dbname+drivername:// protocol is recommended. "
+                    "For MySQL, it is strongly recommended that "
+                    "mysql+pymysql:// "
+                    "be specified for maximum service compatibility",
+
+                ),
+                mock.call.warning(
+                    "URL postgresql://scott:***@some_host/some_db does not "
+                    "contain a '+drivername' portion, "
+                    "and will make use of a default driver.  "
+                    "A full dbname+drivername:// protocol is recommended."
+                )
+            ],
+            warnings.mock_calls
+        )
+
+
+class ProcessGuardTest(db_test_base._DbTestCase):
     def test_process_guard(self):
         self.engine.dispose()
 
@@ -676,23 +850,23 @@ class ProcessGuardTest(test_base.DbTestCase):
 
         with mock.patch("os.getpid", get_parent_pid):
             with self.engine.connect() as conn:
-                dbapi_id = id(conn.connection.connection)
+                dbapi_id = id(compat.driver_connection(conn))
 
         with mock.patch("os.getpid", get_child_pid):
             with self.engine.connect() as conn:
-                new_dbapi_id = id(conn.connection.connection)
+                new_dbapi_id = id(compat.driver_connection(conn))
 
         self.assertNotEqual(dbapi_id, new_dbapi_id)
 
         # ensure it doesn't trip again
         with mock.patch("os.getpid", get_child_pid):
             with self.engine.connect() as conn:
-                newer_dbapi_id = id(conn.connection.connection)
+                newer_dbapi_id = id(compat.driver_connection(conn))
 
         self.assertEqual(new_dbapi_id, newer_dbapi_id)
 
 
-class PatchStacktraceTest(test_base.DbTestCase):
+class PatchStacktraceTest(db_test_base._DbTestCase):
 
     def test_trace(self):
         engine = self.engine
@@ -715,10 +889,28 @@ class PatchStacktraceTest(test_base.DbTestCase):
             with mock.patch.object(engine.dialect, "do_execute") as mock_exec:
 
                 mock_exec.side_effect = orig_do_exec
-                conn.execute("select 1;")
+                conn.execute(sql.text("select 1"))
 
             call = mock_exec.mock_calls[0]
 
             # we're the caller, see that we're in there
             caller = os.path.join("tests", "sqlalchemy", "test_sqlalchemy.py")
             self.assertIn(caller, call[1][1])
+
+
+class MySQLConnectPingListenerTest(db_test_base._MySQLOpportunisticTestCase):
+
+    def test__connect_ping_listener(self):
+        for idx in range(2):
+            with self.engine.begin() as conn:
+                self.assertTrue(isinstance(conn._transaction,
+                                           base_engine.RootTransaction))
+                # TODO(ralonsoh): drop this check once SQLAlchemy minimum
+                #  version is 2.0.
+                if compat.sqla_2:
+                    engines._connect_ping_listener(conn)
+                    self.assertIsNone(conn._transaction)
+                else:
+                    engines._connect_ping_listener(conn, False)
+                    self.assertTrue(isinstance(conn._transaction,
+                                               base_engine.RootTransaction))
